<?php

namespace Etn;

defined( 'ABSPATH' ) || exit;

include_once ABSPATH . 'wp-admin/includes/plugin.php';

/**
 * Plugin final Class.
 * Handles dynamically loading classes only when needed. CheFck Elementor Plugin.
 *
 * @since 1.0.0
 */
final class Plugin {

    private static $instance;
    private $event;
    private $speaker;
    private $schedule;
    private $attendee;

    /**
     * __construct function
     * @since 1.0.0
     */
    public function __construct() {
        // load autoload method
        Autoloader::run();
    }

    /**
     * Public function init.
     * call function for all
     *
     * @since 1.0.0
     */
    public function init() {

        include_once ABSPATH . 'wp-admin/includes/plugin.php';

        //handle woocommerce notice depending on settings
        $this->handle_woo_dependency();

        //handle buy-pro notice
        $this->handle_buy_pro_menu();

        //update plugin meta and plugin actions
        $this->update_plugin_meta_actions();

        // check permission for manage user
        if ( current_user_can( 'manage_options' ) ) {
            add_action( 'admin_menu', [$this, 'register_admin_menu'] );
        }

        //register all styles and scripts
        add_action( 'admin_enqueue_scripts', [$this, 'js_css_admin'] );
        add_action( 'wp_enqueue_scripts', [$this, 'js_css_public'] );
        add_action( 'elementor/frontend/before_enqueue_scripts', [$this, 'elementor_js'] );
        
        // Initialize event module
        $this->event = Core\Event\Base::instance();
        Core\Event\Base::instance()->init();

        // Initialize attendee module
        $this->attendee = Core\Attendee\Base::instance();
        Core\Attendee\Base::instance()->init();

        // Initialize schedule module
        $this->schedule = Core\Schedule\Base::instance();
        Core\Schedule\Base::instance()->init();

        // Initialize speaker module
        $this->speaker = Core\Speaker\Base::instance();
        Core\Speaker\Base::instance()->init();

        // Iinitialize event ticket registration module
        Core\Event\Registration::instance()->init();

        // Initialize attendee information-update module
        Core\Attendee\InfoUpdate::instance()->init();

        // Initialize woocommerce module
        Core\Woocommerce\Base::instance()->init();

        // initialize niche shortcode
        Core\Shortcodes\Hooks::instance()->init();

        // initialize zoom module
        Core\Zoom_Meeting\Hooks::instance()->init();

        // initialize elementor widget
        Widgets\Manifest::instance()->init();

        //make admin menu open if custom taxonomy is selected
        add_action( 'parent_file', [$this, 'keep_taxonomy_menu_open'] );

        // add minicart to header
        add_action('wp_head', [$this, 'etn_custom_inline_css']);

        //initialize banner config
        add_action('admin_head', function() {
                
            \Wpmet\Libs\Banner\Init::instance()
            ->set_text_domain('eventin')
            ->set_api_url('https://api.wpmet.com/public/jhanda/index.php')
            ->set_plugin_screens('edit-etn')
            ->set_plugin_screens('edit-etn-attendee')
            ->set_plugin_screens('edit-etn_category')
            ->set_plugin_screens('edit-etn_tags')
            ->set_plugin_screens('edit-etn-schedule')
            ->set_plugin_screens('edit-etn_speaker_category')
            ->set_plugin_screens('edit-etn-speaker')
            ->set_plugin_screens('eventin_page_etn-event-settings')
            ->set_plugin_screens('eventin_page_etn_sales_report')
            ->set_plugin_screens('edit-etn-zoom-meeting')
            ->call();
            
        });
    }


    /**
     * Handle woocommerce admin notice depending on settings
     *
     * @return void
     */
    public function handle_woo_dependency(){
        
        $eventin_global_settings = \Etn\Utils\Helper::get_settings();
        $sell_tickets            = !empty( $eventin_global_settings["sell_tickets"] ) ? true : false;

        if( $sell_tickets ){
            if ( !is_plugin_active( 'woocommerce/woocommerce.php' ) ) {
                add_action( 'admin_notices', [$this, 'admin_notice_wc_not_active'] );
            }
        }
    }

    /**
     * Show buy-pro menu if pro plugin not found in plugins directory
     *
     * @return void
     */
    public function handle_buy_pro_menu(){
        if ( !file_exists( WP_PLUGIN_DIR . '/eventin-pro/eventin-pro.php' ) ) {
            add_action( "admin_menu", [$this, "add_submenu_for_go_pro"], 11 );
        }
    }

    /**
     * Undocumented function
     *
     * @return void
     */
    public function update_plugin_meta_actions(){
        //Plugins action page links
        add_filter( 'plugin_action_links_' . plugin_basename( ETN_FILE ), [$this, 'plugin_action_links'] );

        //Plugin Row Meta
        add_filter('plugin_row_meta', array($this, 'plugin_row_meta'), 10, 2);
    }

    /**
     * Undocumented function
     *
     * @return void
     */
    public function define_constants(){
        
        define( 'ETN_FILE', __FILE__);
        define( 'ETN_PATH', plugin_dir_url( ETN_FILE ) );
        define( 'ETN_DIR', untrailingslashit( plugin_dir_path( ETN_FILE ) ) );
        define( 'ETN_ASSETS', ETN_PATH . 'assets/' );
        define( 'ETN_CORE', ETN_DIR . '/core/' );
        define( 'ETN_UTILS', ETN_DIR . '/utils/' );
        define( 'ETN_BASENAME', plugin_basename( ETN_FILE ) );
        define( 'ETN_VERSION', '2.1.1' );
    }

    /**
     * Undocumented function
     *
     * @param [type] $parent_file
     * @return void
     */
    public function keep_taxonomy_menu_open( $parent_file ) {
        global $current_screen;
        $taxonomy = $current_screen->taxonomy;

        if ( $taxonomy == 'etn_category' || $taxonomy == 'etn_tags' || $taxonomy == 'etn_speaker_category' ) {
            $parent_file = 'etn-events-manager';
        }

        return $parent_file;
    }

    /**
     * Show notice if woocommerce not active
     *
     * @return void
     */
    public function admin_notice_wc_not_active() {

        if ( file_exists( WP_PLUGIN_DIR . '/woocommerce/woocommerce.php' ) ) {
            $btn['label'] = esc_html__( 'Activate WooCommerce', 'eventin' );
            $btn['url']   = wp_nonce_url( 'plugins.php?action=activate&plugin=woocommerce/woocommerce.php&plugin_status=all&paged=1', 'activate-plugin_woocommerce/woocommerce.php' );
        } else {
            $btn['label'] = esc_html__( 'Install WooCommerce', 'eventin' );
            $btn['url']   = wp_nonce_url( self_admin_url( 'update.php?action=install-plugin&plugin=woocommerce' ), 'install-plugin_woocommerce' );
        }

        Utils\Notice::push(
            [
                'id'          => 'unsupported-woocommerce-version',
                'type'        => 'error',
                'dismissible' => true,
                'btn'         => $btn,
                'message'     => sprintf( esc_html__( 'Eventin requires WooCommerce to get all features, which is currently NOT RUNNING.', 'eventin' ) ),
            ]
        );
    }

    /**
     * Add admin submenu page for go-pro
     *
     * @return void
     */
    public function add_submenu_for_go_pro() {
        //add submenu page for go-pro
        add_submenu_page(
            'etn-events-manager',
            __( 'Get Pro', 'eventin' ),
            __( '<span class="dashicons dashicons-star-filled etn-icon-get-pro"></span> Get Pro', 'eventin' ),
            'edit_posts',
            'edit.php?page=etn-get-pro',
            [$this, 'etn_get_pro']
        );

    }

    /**
     * render go-pro markup
     *
     * @return void
     */
    public function etn_get_pro() {
        //redirect to pro plugin
        $pro_url = add_query_arg( 'etn-dashboard', '1', 'https://themewinter.com/eventin/' );
        wp_redirect( $pro_url );
        exit();
    }

    public function plugin_action_links( $actions ) {

        if ( !file_exists( WP_PLUGIN_DIR . '/eventin-pro/eventin-pro.php' ) ) {
            $actions['etn_pro_link'] = '<a href="https://themewinter.com/eventin/" target="_blank"><span class="etn-text-get-pro">' . __( 'Get Pro', 'eventin' ) . '</span></a>';
        }

        $actions['etn_documentation'] = '<a href="https://support.themewinter.com/docs/plugins/docs/general-settings/" target="_blank">' . __( 'Documentation', 'eventin' ) . '</a>';
        return $actions;
    }
    

	public function plugin_row_meta($plugin_meta, $plugin_file){

        if( $plugin_file === ETN_BASENAME ){
            $plugin_meta[] = sprintf( '<a href="%s" target="_blank">%s</a>',
                esc_url( 'http://support.themewinter.com/support-center/' ),
                __( '<strong class="etn-text-get-pro">Get Support</strong>', 'eventin' )
            );
            
        }

		return $plugin_meta;
	}

    /**
     * Public function version.
     * set for plugin version
     *
     * @since 1.0.0
     */
    public function version() {
        return ETN_VERSION;
    }

    /**
     * Public function package_type.
     * set for plugin package type
     *
     * @since 1.0.0
     */
    public function package_type() {
        return 'free';
    }

    /**
     * Public function plugin_url.
     * set for plugin url
     *
     * @since 1.0.0
     */
    public function plugin_url() {
        return trailingslashit( plugin_dir_url( __FILE__ ) );
    }

    /**
     * Public function plugin_dir.
     * set for plugin dir
     *
     * @since 1.0.0
     */
    public function plugin_dir() {
        return trailingslashit( plugin_dir_path( __FILE__ ) );
    }

    /**
     * Public function core_url .
     * set for plugin  core folder url
     *
     * @since 1.0.0
     */
    public function core_url() {
        return $this->plugin_url() . 'core/';
    }

    /**
     * Public function core_dir .
     * set for plugin  core folder dir
     *
     * @since 1.0.0
     */
    public function core_dir() {
        return $this->plugin_dir() . 'core/';
    }

    /**
     * Public function base_url .
     * set for plugin  base folder url
     *
     * @since 1.0.0
     */
    public function base_url() {
        return $this->plugin_url() . 'base/';
    }

    /**
     * Public function base_dir .
     * set for plugin  base folder dir
     *
     * @since 1.0.0
     */
    public function base_dir() {
        return $this->plugin_dir() . 'base/';
    }

    /**
     * Public function utils_url .
     * set for plugin  utils folder url
     *
     * @since 1.0.0
     */
    public function utils_url() {
        return $this->plugin_url() . 'utils/';
    }

    /**
     * Public function utils_dir .
     * set for plugin  utils folder dir
     *
     * @since 1.0.0
     */
    public function utils_dir() {
        return $this->plugin_dir() . 'utils/';
    }

    /**
     * Public function widgets_url .
     * set for plugin  widget folder url
     *
     * @since 1.0.0
     */
    public function widgets_url() {
        return $this->plugin_url() . 'widgets/';
    }

    /**
     * Public function widgets_dir .
     * set for plugin  widget folder dir
     *
     * @since 1.0.0
     */
    public function widgets_dir() {
        return $this->plugin_dir() . 'widgets/';
    }

    public function text_domain() {
        return 'eventin';
    }

    /**
     * Public function js_css_public .
     * Include public function
     *
     * @since 1.0.0
     */
    public function js_css_public() {

        if ( is_rtl() ) {
            wp_enqueue_style( 'etn-rtl', ETN_ASSETS . 'css/rtl.css' );
        }

        wp_enqueue_style( 'etn-public-css', ETN_ASSETS . 'css/event-manager-public.css', [], $this->version(), 'all' );
        wp_enqueue_style( 'fontawesome', ETN_ASSETS . 'css/font-awesome.css', [], '5.0', 'all' );
        wp_enqueue_script( 'etn-public', ETN_ASSETS . 'js/event-manager-public.js', ['jquery'], $this->version(), true );
    }

    public function elementor_js() {
        wp_enqueue_script( 'etn-elementor-inputs', ETN_ASSETS . 'js/elementor.js', ['elementor-frontend'], $this->version(), true );
    }

    public function js_css_admin() {

        // get screen id
        $screen   = get_current_screen();
        $form_cpt = $this->event;

        if ( !wp_style_is( 'wp-color-picker', 'enqueued' ) ) {
            wp_enqueue_style( 'wp-color-picker' );
        }

        wp_enqueue_style( 'thickbox' );
        wp_enqueue_style( 'select2', ETN_ASSETS . 'css/select2.min.css', [], '4.0.10', 'all' );

        wp_enqueue_style( 'fontawesome', ETN_ASSETS . 'css/font-awesome.css', [], '5.0', 'all' );
        wp_enqueue_style( 'etn-ui', ETN_ASSETS . 'css/etn-ui.css', [], $this->version(), 'all' );
        wp_enqueue_style( 'etn-icon', ETN_ASSETS . 'css/etn-icon.css', [], $this->version(), 'all' );
        wp_enqueue_style( 'jquery-ui', ETN_ASSETS . 'css/jquery-ui.css', ['wp-color-picker'], $this->version(), 'all' );
        wp_enqueue_style( 'admin-datetime', ETN_ASSETS . 'css/jquery.datetimepicker.min.css', [], $this->version(), 'all' );
        wp_enqueue_style( 'event-manager-admin', ETN_ASSETS . 'css/event-manager-admin.css', [], $this->version(), 'all' );

        if ( !did_action( 'wp_enqueue_media' ) ) {
            wp_enqueue_media();
        }

        // js
        wp_enqueue_script( 'wp-color-picker' );
        wp_enqueue_script( 'media-upload' );
        wp_enqueue_script( 'thickbox' );
        wp_enqueue_script( 'jquery-ui-datepicker' );

        wp_enqueue_script( 'jquery-ui', ETN_ASSETS . 'js/etn-ui.min.js', ['jquery'], '4.0.10', true );
        wp_enqueue_script( 'popper', ETN_ASSETS . 'js/Popper.js', ['jquery'], '4.0.10', false );
        wp_enqueue_script( 'etn', ETN_ASSETS . 'js/event-manager-admin.js', ['jquery'], $this->version(), false );
        wp_enqueue_script( 'select2', ETN_ASSETS . 'js/select2.min.js', ['jquery'], '4.0.10', false );
        wp_enqueue_script( 'jquery-repeater', ETN_ASSETS . 'js/jquery.repeater.min.js', ['jquery'], '4.0.10', true );
        wp_enqueue_script( 'admin-datetime', ETN_ASSETS . 'js/jquery.datetimepicker.full.min.js', ['jquery'], $this->version(), true );

        // locallize data
        $settings                                 = \Etn\Core\Event\Settings\Settings::instance()->get_settings_option();
        $form_data                                = [];
        $form_data['ajax_url']                    = admin_url( 'admin-ajax.php' );
        $form_data['zoom_connection_check_nonce'] = wp_create_nonce( 'zoom_connection_check_nonce' );
        $form_data['zoom_module']                 = empty( $settings['etn_zoom_api'] ) ? "no" : "yes";
        $form_data['attendee_module']             = empty( $settings['attendee_registration'] ) ? "no" : "yes";

        wp_localize_script( 'etn', 'form_data', $form_data );
    }

    function register_admin_menu() {
        $admin_page = new \Etn\Core\Event\Pages\Event_Admin_Page();
        $admin_page->add_admin_pages();
        $settings = new Core\Event\Settings( 'etn', $this->version() );
    }

    public function flush_rewrites() {
        $event = new Core\Event\Cpt();
        $event->flush_rewrites();
        $speaker = new Core\Speaker\Cpt();
        $speaker->flush_rewrites();
        $schedule = new Core\Schedule\Cpt();
        $schedule->flush_rewrites();
        
        // flush cpt for attendee
        $attendee = new Core\Attendee\Cpt();
        $attendee->flush_rewrites();
        
        $this->_action_create_table();
    }

    public static function instance() {

        if ( !self::$instance ) {
            self::$instance = new self();
        }

        return self::$instance;
    }

    /**
     * Create plugin specific tables to store data
     *
     * @return void
     */
    public function _action_create_table() {
        global $wpdb;
        $tableName       = $wpdb->prefix . 'etn_events';
        $charset_collate = $wpdb->get_charset_collate();

        // create table for donation
        if ( $wpdb->get_var( "SHOW TABLES LIKE '$tableName'" ) != $tableName ) {

            require_once ABSPATH . 'wp-admin/includes/upgrade.php';

            // create fundraising table
            $wdp_sql = "CREATE TABLE IF NOT EXISTS `$tableName` (
			  `event_id` mediumint(9) NOT NULL AUTO_INCREMENT,
			  `post_id` bigint(20) NOT NULL COMMENT 'This id is teh event id',
			  `form_id` bigint(20) NOT NULL COMMENT 'This id From wp post table',
			  `invoice` varchar(150) NOT NULL,
			  `event_amount` double NOT NULL DEFAULT '0',
			  `user_id` mediumint(9) NOT NULL,
			  `email` varchar(200) NOT NULL,
			  `event_type` ENUM('ticket') DEFAULT 'ticket',
			  `payment_type` ENUM('woocommerce') DEFAULT 'woocommerce',
			  `pledge_id` varchar(20) NOT NULL DEFAULT '0',
			  `payment_gateway` ENUM('offline_payment', 'online_payment', 'bank_payment', 'check_payment', 'stripe_payment', 'other_payment') default 'online_payment',
			  `date_time` datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
			  `status` ENUM('Active', 'Pending', 'Review', 'Refunded', 'DeActive', 'Delete') DEFAULT 'Pending',
			  PRIMARY KEY (`event_id`)
			) $charset_collate;";

            dbDelta( $wdp_sql );

            // create meta table
            $tableNameMeta = $wpdb->prefix . 'etn_trans_meta';

            $wdp_meta = "
				CREATE TABLE IF NOT EXISTS `$tableNameMeta`(
					`meta_id` mediumint NOT NULL AUTO_INCREMENT,
					`event_id` mediumint NOT NULL,
					`meta_key` varchar(255),
					`meta_value` longtext,
					PRIMARY KEY(`meta_id`)
				) $charset_collate;
			";
            dbDelta( $wdp_meta );

            update_option( 'etn_version', ETN_VERSION );
        }

    }

    
    /**
     * Custom inline css
     *
     */
    public function etn_custom_inline_css(){
      $settings =  \Etn\Core\Event\Settings\Settings::instance()->get_settings_option();
      $etn_custom_css = '';
      $primary_color = "#5D78FF";
      $secondary_color = "";

        // cart bg color
        if( !empty( $settings['etn_primary_color'] ) ){
        $primary_color =  $settings['etn_primary_color'] ;
        } 

        // cart icon color
        if( !empty( $settings['etn_secondary_color'] ) ){
        $secondary_color = $settings['etn_secondary_color'] ;
        }

      $etn_custom_css.="
        .etn-event-single-content-wrap .etn-event-meta .etn-event-category span,
        .etn-event-item .etn-event-footer .etn-atend-btn .etn-btn-border,
        .etn-btn.etn-btn-border, .attr-btn-primary.etn-btn-border, 
        .etn-attendee-form .etn-btn.etn-btn-border, 
        .etn-ticket-widget .etn-btn.etn-btn-border,
        .etn-settings-dashboard .button-primary.etn-btn-border,
        .etn-single-speaker-item .etn-speaker-content a:hover,
        .etn-event-style2 .etn-event-date,
        .etn-event-style3 .etn-event-content .etn-title a:hover,
        .etn-event-item:hover .etn-title a{
            color: {$primary_color}; 
        }
        .etn-event-item .etn-event-category span,
        .etn-btn, .attr-btn-primary, 
        .etn-attendee-form .etn-btn, 
        .etn-ticket-widget .etn-btn,
        .schedule-list-1 .schedule-header,
        .speaker-style4 .etn-speaker-content .etn-title a,
        .etn-speaker-details3 .speaker-title-info,
        .etn-event-slider .swiper-pagination-bullet, .etn-speaker-slider .swiper-pagination-bullet,
        .etn-event-slider .swiper-button-next, .etn-event-slider .swiper-button-prev,
         .etn-speaker-slider .swiper-button-next, .etn-speaker-slider .swiper-button-prev,
        .etn-single-speaker-item .etn-speaker-thumb .etn-speakers-social a,
        .etn-event-header .etn-event-countdown-wrap .etn-count-item, 
        .schedule-tab-1 .etn-nav li a.etn-active,
        .etn-settings-dashboard .button-primary{
            background-color: {$primary_color}; 
        }

        .etn-event-item .etn-event-footer .etn-atend-btn .etn-btn-border,
        .etn-btn.etn-btn-border, .attr-btn-primary.etn-btn-border,
        .etn-attendee-form .etn-btn.etn-btn-border,
        .etn-ticket-widget .etn-btn.etn-btn-border,
        .etn-settings-dashboard .button-primary.etn-btn-border{
            border-color: {$primary_color}; 
        }
        .schedule-tab-wrapper .etn-nav li a.etn-active{
            border-bottom-color: {$primary_color}; 
        }
        .schedule-tab-wrapper .etn-nav li a:after,
        .schedule-tab-1 .etn-nav li a.etn-active:after{
            border-color: {$primary_color} transparent transparent transparent;
        }
        
        .etn-event-item .etn-event-location,
        .etn-event-tag-list a:hover,
        .etn-schedule-wrap .etn-schedule-info .etn-schedule-time{
            color: {$secondary_color}; 
        }
        .etn-event-tag-list a:hover{
            border-color: {$secondary_color}; 
        }
        .etn-btn:hover, .attr-btn-primary:hover,
        .etn-attendee-form .etn-btn:hover,
        .etn-ticket-widget .etn-btn:hover,
        .speaker-style4 .etn-speaker-content p,
        .etn-single-speaker-item .etn-speaker-thumb .etn-speakers-social a:hover,
        .etn-settings-dashboard .button-primary:hover{
            background-color: {$secondary_color}; 
        }

      
      


      ";

      // add inline css
      wp_register_style('etn-custom-css', false);
      wp_enqueue_style('etn-custom-css');
      wp_add_inline_style('etn-custom-css', $etn_custom_css);
  }

}
