<?php

namespace Etn\Utils;

use Etn\Utils\Helper as Helper;

defined( 'ABSPATH' ) || exit;

class Utilities {

    /**
     * checks if attachment is an image
     *
     * @param [type] $attachment_id
     * @return void
     */
    public static function event_attachment_type_is_image( $attachment_id = null ) {

        if ( is_null( $attachment_id ) || $attachment_id == '' ) {
            return false;
        }

        $path = wp_get_attachment_url( $attachment_id );

        if ( $path == '' ) {
            return false;
        }

        $image      = getimagesize( $path );
        $image_type = $image[2];

        if ( in_array( $image_type, [IMAGETYPE_GIF, IMAGETYPE_JPEG, IMAGETYPE_PNG, IMAGETYPE_BMP] ) ) {
            return true;
        }

        return false;
    }

    /**
     * sanitizes given input
     *
     * @param string $data
     * @return void
     */
    public static function sanitize( string $data ) {
        return strip_tags(
            stripslashes(
                sanitize_text_field(
                    filter_input( INPUT_POST, $data )
                )
            )
        );
    }

    /**
     * returns list of all speaker
     * returns single speaker if speaker id is provuded
     *
     * @param [type] $id
     * @return void
     */
    public static function get_speakers( $id = null ) {
        $return_organizers = [];
        try {

            if ( is_null( $id ) ) {
                $args = [
                    'post_type'      => 'etn-speaker',
                    'post_status'    => 'publish',
                    'posts_per_page' => -1,
                ];
                $organizers = get_posts( $args );

                foreach ( $organizers as $value ) {
                    $return_organizers[$value->ID] = $value->post_title;
                }

                return $return_organizers;
            } else {
                // return single speaker
                return get_post( $id );
            }

        } catch ( \Exception $es ) {
            return [];
        }

    }

    /**
     * returns category of a speaker
     *
     * @param [type] $id
     * @return void
     */
    public static function get_speakers_category( $id = null ) {
        $speaker_category = [];
        try {

            if ( is_null( $id ) ) {
                $terms = get_terms( [
                    'taxonomy'   => 'etn_speaker_category',
                    'hide_empty' => false,
                ] );

                foreach ( $terms as $speakers ) {
                    $speaker_category[$speakers->term_id] = $speakers->name;
                }

                return $speaker_category;
            } else {
                // return single speaker
                return get_post( $id );
            }

        } catch ( \Exception $es ) {
            return [];
        }

    }

    /**
     * returns category of an event
     *
     * @param [type] $id
     * @return void
     */
    public static function get_event_category( $id = null ) {
        $event_category = [];
        try {

            if ( is_null( $id ) ) {
                $terms = get_terms( [
                    'taxonomy'   => 'etn_category',
                    'hide_empty' => false,
                ] );

                foreach ( $terms as $event ) {
                    $event_category[$event->term_id] = $event->name;
                }

                return $event_category;
            } else {
                // return single speaker
                return get_post( $id );
            }

        } catch ( \Exception $es ) {
            return [];
        }

    }

    /**
     * returns tag of an event
     *
     * @param [type] $id
     * @return void
     */
    public static function get_event_tag( $id = null ) {
        $event_tag = [];
        try {

            if ( is_null( $id ) ) {
                $terms = get_terms( [
                    'taxonomy'   => 'etn_tags',
                    'hide_empty' => false,
                ] );

                foreach ( $terms as $event ) {
                    $event_tag[$event->term_id] = $event->name;
                }

                return $event_tag;
            } else {
                // return single speaker
                return get_post( $id );
            }

        } catch ( \Exception $es ) {
            return [];
        }

    }

    public static function get_schedules( $id = null ) {
        $return_schedules = [];
        try {

            if ( is_null( $id ) ) {
                $args = [
                    'post_type'      => 'etn-schedule',
                    'post_status'    => 'publish',
                    'posts_per_page' => -1,
                ];
                $schedules = get_posts( $args );

                foreach ( $schedules as $value ) {
                    $schedule_date                = get_post_meta( $value->ID, 'etn_schedule_date', true );
                    $return_schedules[$value->ID] = $value->post_title . " ($schedule_date)";
                }

                return $return_schedules;
            } else {
                // return single speaker
                return get_post( $id );
            }

        } catch ( \Exception $es ) {
            return [];
        }

    }

    public static function get_events( $id = null ) {
        $return_events = [];
        try {

            if ( is_null( $id ) ) {
                $args = [
                    'post_type'      => 'etn',
                    'post_status'    => 'publish',
                    'posts_per_page' => -1,
                ];
                $events = get_posts( $args );

                foreach ( $events as $value ) {
                    $return_events[$value->ID] = $value->post_title;
                }

                return $return_events;
            } else {
                // return single speaker
                return get_post( $id );
            }

        } catch ( \Exception $es ) {
            return [];
        }

    }

    public static function get_users( $id = null ) {
        $return_organizers = ['' => esc_html__( 'select organizer', 'eventin' )];
        try {
            $blogusers = get_users(
                [
                    'order'    => 'DESC',
                    'role__in' => ['etn_organizer', 'administrator'],
                ]
            );

            foreach ( $blogusers as $user ) {
                $name                         = isset( $user->display_name ) ? $user->display_name : $user->user_nicename;
                $return_organizers[$user->ID] = $name . ' - ' . $user->user_email;
            }

            return $return_organizers;
        } catch ( \Exception $es ) {
            return [];
        }

    }

    public static function user_can_access( $cap = null ) {
        include_once ABSPATH . 'wp-includes/pluggable.php';

        if ( current_user_can( $cap ) ) {
            return true;
        }

        return false;
    }

    public static function etn_event_manager_fontawesome_icons( $prefix = 'fab' ) {
        $prefix       = apply_filters( 'etn_event_social_icons_prefix', $prefix );
        $social_icons = [
            "$prefix fa-facebook"           => esc_html__( 'facebook', 'eventin' ),
            "$prefix fa-facebook-f"         => esc_html__( 'facebook-f', 'eventin' ),
            "$prefix fa-facebook-messenger" => esc_html__( 'facebook-messenger', 'eventin' ),
            "$prefix fa-facebook-square"    => esc_html__( 'facebook-square', 'eventin' ),
            "$prefix fa-linkedin"           => esc_html__( 'linkedin', 'eventin' ),
            "$prefix fa-linkedin-in"        => esc_html__( 'linkedin-in', 'eventin' ),
            "$prefix fa-twitter"            => esc_html__( 'twitter', 'eventin' ),
            "$prefix fa-twitter-square"     => esc_html__( 'twitter-square', 'eventin' ),
            "$prefix fa-uber"               => esc_html__( 'uber', 'eventin' ),
            "$prefix fa-google"             => esc_html__( 'google', 'eventin' ),
            "$prefix fa-google-drive"       => esc_html__( 'google-drive', 'eventin' ),
            "$prefix fa-google-play"        => esc_html__( 'google-play', 'eventin' ),
            "$prefix fa-google-wallet"      => esc_html__( 'google-wallet', 'eventin' ),
            "$prefix fa-linkedin"           => esc_html__( 'linkedin', 'eventin' ),
            "$prefix fa-linkedin-in"        => esc_html__( 'linkedin-in', 'eventin' ),
            "$prefix fa-whatsapp"           => esc_html__( 'whatsapp', 'eventin' ),
            "$prefix fa-whatsapp-square"    => esc_html__( 'whatsapp-square', 'eventin' ),
            "$prefix fa-wordpress"          => esc_html__( 'wordpress', 'eventin' ),
            "$prefix fa-wordpress-simple"   => esc_html__( 'wordpress-simple', 'eventin' ),
            "$prefix fa-youtube"            => esc_html__( 'youtube', 'eventin' ),
            "$prefix fa-youtube-square"     => esc_html__( 'youtube-square', 'eventin' ),
            "$prefix fa-xbox"               => esc_html__( 'xbox', 'eventin' ),
            "$prefix fa-vk"                 => esc_html__( 'vk', 'eventin' ),
            "$prefix fa-vnv"                => esc_html__( 'vnv', 'eventin' ),
            "$prefix fa-instagram"          => esc_html__( 'instagram', 'eventin' ),
            "$prefix fa-reddit"             => esc_html__( 'reddit', 'eventin' ),
            "$prefix fa-reddit-alien"       => esc_html__( 'reddit-alien', 'eventin' ),
            "$prefix fa-reddit-square"      => esc_html__( 'reddit-square', 'eventin' ),
            "$prefix fa-pinterest"          => esc_html__( 'pinterest', 'eventin' ),
            "$prefix fa-pinterest-p"        => esc_html__( 'pinterest-p', 'eventin' ),
            "$prefix fa-pinterest-square"   => esc_html__( 'pinterest-square', 'eventin' ),
            "$prefix fa-tumblr"             => esc_html__( 'tumblr', 'eventin' ),
            "$prefix fa-tumblr-square"      => esc_html__( 'tumblr-square', 'eventin' ),
            "$prefix fa-flickr"             => esc_html__( 'flickr', 'eventin' ),
            "$prefix fa-meetup"             => esc_html__( 'meetup', 'eventin' ),
            "$prefix fa-share"              => esc_html__( 'share', 'eventin' ),
            "$prefix fa-vimeo-v"            => esc_html__( 'vimeo', 'eventin' ),
            "$prefix fa-weixin"             => esc_html__( 'Wechat', 'eventin' ),
        ];
        return apply_filters( 'etn_social_icons', $social_icons );
    }

    /**
     * returns list of organizers'
     * returns single organizer if organizer title is passed as argument
     *
     * @param [type] $title
     * @return void
     */
    public static function get_organizers( $title = null ) {
        global $wpdb;
        $return_organizers = [];
        try {

            if ( is_null( $title ) ) {
                $orgs = $wpdb->get_results( "SELECT * FROM $wpdb->postmeta WHERE meta_key = 'etn_event_organizer'", ARRAY_A );

                foreach ( $orgs as $org ) {
                    $org_data                                               = unserialize( $org['meta_value'] );
                    $return_organizers[$org_data[0]["etn_organizer_title"]] = $org_data[0]["etn_organizer_title"];
                }

                return $return_organizers;
            } else {
                // return single organizer
                $org      = $wpdb->get_results( "SELECT * FROM $wpdb->postmeta WHERE meta_key = 'etn_event_organizer' AND  meta_value LIKE '%$title%' LIMIT 1", ARRAY_A );
                $org_data = unserialize( $org[0]['meta_value'] );
                return $org_data[0];
            }

        } catch ( \Exception $es ) {
            return [];
        }

    }

    /**
     * returns all organizers list
     *
     * @return void
     */
    public static function get_orgs() {
        $return_organizers = [];
        try {
            $terms = get_terms( [
                'taxonomy'   => 'etn_speaker_category',
                'orderby'    => 'count',
                'hide_empty' => false,
                'fields'     => 'all',
            ] );

            foreach ( $terms as $term ) {
                $return_organizers[$term->slug] = $term->name;
            }
            return $return_organizers;
        } catch ( \Exception $es ) {
            return [];
        }

    }

    /**
     * returns all categories of an event
     *
     * @param [type] $post_id
     * @param [type] $category
     * @param boolean $single
     * @return void
     */
    public static function cate_with_link( $post_id = null, $category, $single = false ) {
        $terms         = get_the_terms( $post_id, $category );
        $category_name = '';

        if ( is_array( $terms ) ):

            foreach ( $terms as $tkey => $term ):
                $cat = $term->name;

                $category_name .= sprintf( "<span>%s</span> ", $cat );

                if ( $single ) {
                    break;
                }

                if ( $tkey == 1 ) {
                    break;
                }

            endforeach;
        endif;
        return $category_name;
    }

    /**
     * validation for nonce
     *
     * @param [type] $nonce_field
     * @param [type] $action
     * @param [type] $post
     * @return boolean
     */
    public static function is_secured( $nonce_field, $action, $post_id = null, $post ) {
        
        $nonce = isset( $post[$nonce_field] ) ? sanitize_text_field( $post[$nonce_field] ) : '';

        if ( $nonce == '' ) {
            return false;
        }

        if( null !== $post_id ){

            if ( !current_user_can( 'edit_post', $post_id ) ) {
                return false;
            }
    
            if ( wp_is_post_autosave( $post_id ) ) {
                return false;
            }
    
            if ( wp_is_post_revision( $post_id ) ) {
                return false;
            }
        }

        if ( !wp_verify_nonce( $nonce, $action ) ) {
            return false;
        }

        return true;
    }
    /**
     * Single page settings option
     *
     */
    public static function single_template_options(){
        $data=[];
        $date_options          = Helper::get_date_formats();
        $text_domain           = 'eventin';
        $etn_start_date        = strtotime(get_post_meta(get_the_id(), 'etn_start_date', true));
        $etn_start_time        = strtotime(get_post_meta(get_the_id(), 'etn_start_time', true));
        $etn_event_location    = get_post_meta(get_the_id(), 'etn_event_location', true);
        $etn_event_tags        = get_post_meta(get_the_id(), 'etn_event_tags', true);
        $etn_event_description = get_post_meta(get_the_id(), 'etn_event_description', true);
        $etn_event_schedule    = get_post_meta(get_the_id(), 'etn_event_schedule', true);
        $etn_online_event      = get_post_meta(get_the_id(), 'etn_online_event', true);
        $etn_es_event_feature  = get_post_meta(get_the_id(), 'etn_es_event_feature', true);
        $etn_event_banner      = get_post_meta(get_the_id(), 'etn_event_banner', true);
        $etn_event_banner_url  = wp_get_attachment_image_src($etn_event_banner);
        $etn_organizer_banner  = get_post_meta(get_the_id(), 'etn_organizer_banner', true);
        $etn_organizer_banner_url = wp_get_attachment_image_src($etn_organizer_banner);
        $etn_end_date          = strtotime(get_post_meta(get_the_id(), 'etn_end_date', true));
        $etn_end_time          = strtotime(get_post_meta(get_the_id(), 'etn_end_time', true));
        $etn_event_socials     = get_post_meta(get_the_id(), 'etn_event_socials', true);
        $etn_event_page        = get_post_meta(get_the_id(), 'etn_event_page', true);
        $etn_organizer_events  = get_post_meta(get_the_id(), 'etn_event_organizer', true);
        $etn_avaiilable_tickets = get_post_meta(get_the_id(), 'etn_avaiilable_tickets', true);
        $etn_avaiilable_tickets = isset($etn_avaiilable_tickets) ? (intval($etn_avaiilable_tickets)) : 0;
        $etn_ticket_unlimited   = get_post_meta(get_the_id(), 'etn_ticket_availability', true);
    
        $cart_product_id = get_post_meta(get_the_id(), 'link_wc_product', true) ? esc_attr(get_post_meta(get_the_id(), 'link_wc_product', true)) : esc_attr(get_the_id());
    
        $etn_sold_tickets  = get_post_meta(get_the_id(), 'etn_sold_tickets', true);
        if (!$etn_sold_tickets) {
            $etn_sold_tickets = 0;
        }
        $etn_ticket_price  = get_post_meta(get_the_id(), 'etn_ticket_price', true);
        $etn_ticket_price  = isset($etn_ticket_price) ? (floatval($etn_ticket_price)) : 0;
        $etn_left_tickets  = $etn_avaiilable_tickets - $etn_sold_tickets;
        $event_options     = get_option("etn_event_options");
        $event_time_format = !isset($event_options["time_format"])  ?   '12' : $event_options["time_format"];
        $event_start_time  = ($event_time_format == "24" || $event_time_format == "") ? date('H:i', $etn_start_time) : date('h:i A', $etn_start_time);
        $event_end_time    = ($event_time_format == "24" || $event_time_format == "") ? date('H:i', $etn_end_time) : date('h:i A', $etn_end_time);
        $event_start_date  = (isset($event_options["date_format"]) && $event_options["date_format"] !== '') ? date($date_options[$event_options["date_format"]], $etn_start_date) : date('d/m/Y', $etn_start_date);
        $event_end_date = '';
        if ($etn_end_date) {
            $event_end_date = isset($event_options["date_format"]) && ("" != $event_options["date_format"]) ? date($date_options[$event_options["date_format"]], $etn_end_date) : date('d/m/Y', $etn_end_date);
        }
        $etn_deadline       =  strtotime(get_post_meta(get_the_id(), 'etn_registration_deadline', true));
        $etn_deadline_value = '';
        if ($etn_deadline) {
            $etn_deadline_value =  isset($event_options["date_format"]) && $event_options["date_format"] !== '' ? date($date_options[$event_options["date_format"]], $etn_deadline) : date('d/m/Y', $etn_deadline);
        }
        $category          =  Utilities::cate_with_link(get_the_ID(), 'etn_category');

        $data['category']               = $category;
        $data['etn_event_schedule']     = $etn_event_schedule;
        $data['event_options']          = $event_options;
        $data['text_domain']            = $text_domain;
        $data['event_start_date']       = $event_start_date;
        $data['event_end_date']         = $event_end_date;
        $data['event_start_time']       = $event_start_time;
        $data['event_end_time']         = $event_end_time;
        $data['etn_deadline_value']     = $etn_deadline_value;
        $data['etn_event_location']     = $etn_event_location;
        $data['etn_left_tickets']       = $etn_left_tickets;
        $data['etn_organizer_events']   = $etn_organizer_events;
        $data['date_options']           = $date_options;
        $data['etn_event_socials']      = $etn_event_socials;
        $data['etn_ticket_price']       = $etn_ticket_price;
        $data['etn_ticket_unlimited']   = $etn_ticket_unlimited;
        return $data;
    }

    /**
     * Single page organizer
     */
    public static function single_template_organizer( $etn_organizer_events ){
        ?>
        <div class="etn-widget etn-event-organizers">
            <h4 class="etn-widget-title etn-title"><?php echo esc_html__("Organizers", "eventin"); ?> </h4>
            <?php
            $term_details = get_term_by('slug',  $etn_organizer_events, 'etn_speaker_category');
            
            if(isset($term_details) && "" != $term_details){
                $organizer_term_id = $term_details->term_id;

                $data = Helper::post_data_query('etn-speaker', -1 , 'DESC' , [$organizer_term_id] , 'etn_speaker_category' );
    
                if (isset( $data ) && !empty( $data )) {
                    foreach ( $data as $value ) { 
                        $social = get_post_meta( $value->ID , 'etn_speaker_socials', true);
                        $email = get_post_meta( $value->ID , 'etn_speaker_website_email', true);
                        $etn_speaker_company_logo = get_post_meta( $value->ID , 'etn_speaker_company_logo', true);
                        $logo = wp_get_attachment_image_src($etn_speaker_company_logo, 'full');
                        ?>
                        <div class="etn-organaizer-item">
                            <?php if (isset($logo[0])) { ?>
                                <div class="etn-organizer-logo">
                                    <img src="<?php echo esc_url($logo[0]); ?>" alt="<?php the_title_attribute(); ?>">
                                </div>
                            <?php } ?>
                            <h4 class="etn-organizer-name">
                                <?php echo esc_html( get_the_title( $value->ID ) ); ?>
                            </h4>
    
                            <?php if ($email) { ?>
                                <div class="etn-organizer-email">
                                    <span class="etn-label-name"><?php echo esc_html__('Email :', "eventin"); ?></span>
                                    <?php echo esc_html($email); ?>
                                </div>
                            <?php } ?>
                            <div class="etn-social">
                                <span class="etn-label-name"><?php echo esc_html__('Social :', "eventin"); ?></span>
                                <?php if (is_array( $social ) && !empty( $social )) { ?>
                                    <?php foreach ($social as $social_value) {  ?>
                                        <?php $etn_social_class = 'etn-' . str_replace('fab fa-', '', $social_value['icon']); ?>

                                        <a href="<?php echo esc_url($social_value["etn_social_url"]); ?>" target="_blank" class="<?php echo esc_attr($etn_social_class); ?>" title="<?php echo esc_attr($social_value["etn_social_title"]); ?>"><i class="<?php echo esc_attr($social_value["icon"]); ?>"></i></a>
                                    <?php  } ?>
                                <?php } ?>
                            </div>
                        </div>
                    <?php  }
                }
            }
        ?>
        </div>
        <?php
    }

    /**
     * Speaker sessions in single page 
     */
    public static function speaker_sessions( $speaker_id ) {
        global $wpdb;
        $orgs = $wpdb->get_results("SELECT * FROM $wpdb->postmeta WHERE meta_key = 'etn_schedule_topics' AND  meta_value LIKE '%$speaker_id%'", ARRAY_A);
        
        return $orgs;
    }

    /**
     * Remove attendee data when status faild
     */
    public static function remove_attendee_data(){
        global $wpdb;
        $query = $wpdb->query( 
            " DELETE etn_postmeta , etn_post
            FROM  $wpdb->posts etn_post
        INNER 
            JOIN $wpdb->postmeta etn_postmeta
            ON  etn_postmeta.post_id = etn_post.ID
        WHERE etn_postmeta.meta_key = 'etn_status' 
            AND etn_postmeta.meta_value = 'failed'"
        );
        return $query;
    }

    /**
     * get  corn shcedule days
     */
    public static function get_schedule_days(){
        // attendee_remove
        $event_options      = get_option("etn_event_options");
        $attendee_remove    = isset($event_options['attendee_remove']) && $event_options['attendee_remove'] !=="" ?  $event_options['attendee_remove'] : 1 ;
        
        return 60 * 60 * 24 * $attendee_remove;
    }
    
    /**
     * Send email function
     */
    public static function send_email( $to, $subject, $mail_body, $from, $from_name ) {
        $body = html_entity_decode($mail_body);
        $headers   = ['Content-Type: text/html; charset=UTF-8', 'From: ' . $from_name . ' <' . $from . '>'];
        $result = wp_mail( $to, $subject, $body, $headers );

        return $result;
    }

}
